/*
 * Adds a button on the Gnome-Shell top bar to launch System Monitor, raise the
 * System Monitor window to the top, or move it to the current desktop.
 *
 * This extension rrquires the xdotool package.
 * Use 'sudo apt install xdotool' to install it.
 *
 */

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Main = imports.ui.main;
const St = imports.gi.St;

let button;

/*
 * Performs three operations:
 * 1. If the System Monitor is not running, System Monitor is started on the
 *    user's current desktop.
 * 2. If the System Monitor is already running and is on the user's current
 *    desktop, then System Monitor is placed as the top window.
 * 3. If the System Monitor is already running and is not on the user's current
 *    desktop, then System Monitor is placed on the user's current desktop.
 */
function _launch() {

    try {
        let window_id = parseInt(GLib.spawn_command_line_sync(
            "xdotool search --onlyvisible --name 'System Monitor'")[1]);
        if (isNaN(window_id)) {
            let application = Gio.DesktopAppInfo.new_from_filename(
                '/usr/share/applications/gnome-system-monitor.desktop');
            application.launch([], null);
        } else {
            let current_desktop_id = parseInt(GLib.spawn_command_line_sync(
                "xdotool get_desktop")[1]);
            let window_desktop_id = parseInt(GLib.spawn_command_line_sync(
                "xdotool get_desktop_for_window " + window_id)[1]);
            if (window_desktop_id == current_desktop_id) {
                /*
                 * If the window is already on the desktop, windowactivate will
                 * place the window as the top window.
                 */
                GLib.spawn_command_line_sync("xdotool windowactivate " +
                    window_id);
            } else {
                /*
                 * If the window is on a different desktop, use windowfocus,
                 * because windowactivate changes the user's desktop to the
                 * window's previous desktop (even even after calling
                 * set_desktop_for_window).
                 *
                 * If the window had previously been on the current desktop,
                 * windowfocus will place the window at its previous level.  In
                 * other words, the window will be placed behind the windows it
                 * was previously behind, and it will be placed in front of the
                 * windows it was previously in front of.  If the window is new
                 * to a desktop, it will be placed in front of existing windows.
                 */
                GLib.spawn_command_line_sync("xdotool set_desktop_for_window " +
                    window_id + " " + current_desktop_id);
                GLib.spawn_command_line_sync("xdotool windowfocus " +
                    window_id);
            }
        }
    } catch(e) {
        log(e);
    }
}

/*
 * Creates the new button, assigns the icon, and connects the button press
 * signal to the handler.
 */
function init() {
    button = new St.Bin({ style_class: 'panel-button',
                          reactive: true,
                          can_focus: true,
                          x_fill: true,
                          y_fill: false,
                          track_hover: true });
    let icon = new St.Icon({ icon_name: 'utilities-system-monitor-symbolic',
        style_class: 'system-status-icon' });
    button.set_child(icon);
    button.connect('button-press-event', _launch);
}

/*
 * Adds the new button to the Gnome-Shell top bar.
 */
function enable() {
    Main.panel._rightBox.insert_child_at_index(button, 0);
}

/*
 * Removes new button to the Gnome-Shell top bar.
 */
function disable() {
    Main.panel._rightBox.remove_child(button);
}

